import { StoreModule } from '../StoreModule/StoreModule';
export declare type IId = string | number;
export interface IWithId {
    id: IId;
}
export interface ICRUDApi {
    add: string;
    list: string;
    get: string;
    edit: string;
    delete: string;
}
export declare abstract class AbstractCRUDRepository<T extends IWithId> extends StoreModule {
    api: ICRUDApi;
    readonly hasList: boolean;
    readonly isListEmpty: boolean;
    readonly isLocalListEmpty: boolean;
    readonly localList: T[];
    readonly addedIndex: (instance: T) => number;
    readonly deletedIndex: (instance: T) => number;
    readonly deletedIndexById: (id: string | number) => number;
    readonly editedIndex: (instance: T) => number;
    readonly editedIndexById: (id: string | number) => number;
    readonly listFirstItem: T;
    readonly listLastItem: T;
    readonly listLastIndex: number;
    readonly listLength: number;
    readonly listItemByIndex: (index: number) => T;
    readonly listItemById: (id: string | number) => T;
    readonly listIndex: (instance: T) => number;
    readonly listIndexById: (id: string | number) => number;
    readonly first: T;
    readonly last: T;
    readonly localListFirstItem: T;
    readonly localListLastItem: T;
    readonly localListLastIndex: number;
    readonly localListLength: number;
    readonly localListItemByIndex: (index: number) => T;
    readonly localListItemById: (id: string | number) => T;
    readonly localListIndex: (instance: T) => number;
    readonly localListIndexById: (id: string | number) => number;
    readonly deletedMap: import("lodash").Dictionary<T>;
    readonly addedMap: import("lodash").Dictionary<T>;
    readonly editedMap: import("lodash").Dictionary<T>;
    readonly listMap: import("lodash").Dictionary<T>;
    readonly localListMap: import("lodash").Dictionary<T>;
    readonly hasAdded: boolean;
    readonly isAdded: (instance: T) => boolean;
    readonly addedById: (id: string | number) => T;
    readonly isAddedById: (id: string | number) => boolean;
    readonly hasEdited: boolean;
    readonly isEdited: (instance: T) => boolean;
    readonly editedById: (id: string | number) => T;
    readonly isEditedById: (id: string | number) => boolean;
    readonly hasDeleted: boolean;
    readonly isDeleted: (instance: T) => boolean;
    readonly deletedById: (id: string | number) => T;
    readonly isDeletedById: (id: string | number) => boolean;
    readonly hasActive: boolean;
    readonly isActive: (instance: T) => boolean;
    readonly isActiveNew: boolean;
    readonly isActiveById: (id: string | number) => boolean;
    readonly listIds: Array<IId>;
    readonly localListIds: Array<IId>;
    readonly isActiveChangedAfterActivation: boolean;
    added: T[];
    edited: T[];
    deleted: T[];
    list: T[] | undefined;
    active: T;
    activeInitialCopy: T | undefined;
    constructor(api: ICRUDApi);
    activate(instance: T): this;
    deactivate(): this;
    activateFirst(): this;
    activateLast(): this;
    activateByIndex(index: number): this;
    activateInstead(insteadInstance: T, instanceToActivate: T): this;
    activateInsteadById(id: string | number, instanceToActivate: T): this;
    deactivateIfActive(instance: T): this;
    resetActiveToInitialState(): this;
    setList(list: T[]): this;
    addLocal(instance: T): this;
    deleteAdded(instance: T): this;
    deleteAllAdded(): this;
    editLocal(instance: T): this;
    deleteEdited(instance: T): this;
    deleteAllEdited(): this;
    deleteLocal(instance: T): this;
    deleteActiveLocal(): this;
    deleteDeleted(instance: T): this;
    deleteAllDeleted(): this;
    clear(): this;
    clearList(): this;
    sendUpsertReq(instance: T, ...params: any[]): Promise<any>;
    abstract sendAddReq(instance: T, ...params: any[]): Promise<T>;
    abstract sendEditReq(instance: T, ...params: any[]): Promise<T>;
    abstract sendDeleteReq(instance: T, ...params: any[]): Promise<void>;
    abstract sendListReq(...params: any[]): Promise<T[]>;
    abstract sendGetReq(id: IId, ...params: any[]): Promise<T>;
    flushAdded(instance: T, ...params: any[]): Promise<T>;
    flushAllAdded(...params: any[]): Promise<Array<T>>;
    flushEdited(instance: T, ...params: any[]): Promise<T>;
    flushAllEdited(...params: any[]): Promise<Array<T>>;
    flushDeleted(instance: T, ...params: any[]): Promise<void>;
    flushAllDeleted(...params: any[]): Promise<Array<void>>;
    flush(): Promise<void>;
    add(instance: T, ...params: any[]): Promise<T>;
    addAndActivate(instance: T, ...params: any[]): Promise<T>;
    edit(instance: T, ...params: any[]): Promise<T>;
    upsert(instance: T, ...params: any[]): Promise<T>;
    upsertAndActivate(instance: T): Promise<T>;
    upsertActive(): Promise<T | undefined>;
    delete(instance: T, ...params: any[]): Promise<void>;
    deleteActive(...params: any[]): Promise<void>;
    deleteByIndex(index: number, ...params: any[]): Promise<void>;
    smartUpsert(instance: T, ...params: any[]): Promise<T>;
    smartUpsertActive(...params: any[]): Promise<T>;
    smartDelete(instance: T, ...params: any[]): Promise<void>;
    smartDeleteActive(): Promise<void>;
    smartUpsertAndActivate(instance: T): Promise<T>;
    updateList(...params: any[]): Promise<T[]>;
    getAndActivate(id: IId, ...params: any[]): Promise<T>;
    refreshItem(instance: T, ...params: any[]): Promise<T>;
    refreshItemById(id: IId): Promise<T>;
    protected getLocalList(): T[];
    protected addToList(instance: T): this;
    protected editInList(instance: T): this;
    protected deleteFromList(instance: T): this;
    protected deleteActiveFromList(): this;
    protected smartReplace(replaceableInstance: T, replacingInstance: T): this;
    protected replaceAdded(replaceableInstance: T, replacingInstance: T): this;
    protected replaceEdited(replaceableInstance: T, replacingInstance: T): this;
    protected replaceDeleted(replaceableInstance: T, replacingInstance: T): this;
    protected replaceInList(replaceableInstance: T, replacingInstance: T): this;
    protected setActiveInitialCopy(instance: T): this;
    protected clearActiveInitialCopy(): this;
    protected throwErrorIfHasntList(): void;
    protected throwErrorIfHasntActive(): void;
}
